#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <SDL/SDL.h>
#include "isola.h"
#include "fct.c"
#include "jeu.c"
#include "ia.c"
#include "menu.c"


// ****************************************************************************
// ********************************** Main ************************************
// ****************************************************************************
int main (int argc, char *argv[])
{
	JEU jeu;					// Donnes principales du jeu
	Uint32 flags;				// Paramtres pour l'affichage (fentre ou plein cran)
	int i;
	
	srand ((unsigned) time (NULL));	// Initialise le gnrateur de nbs alatoires
	flags = SDL_SWSURFACE | SDL_FULLSCREEN;	// Plein cran par dfaut
	
	// Lit les options passes en argument au programme
	for (i = 1; i < argc; i ++)
	{
		if (!strcmp (argv[i], "-h"))		// Affiche une aide
		{	printf ("******** Do'SSi Zo'la *********\nCyril CLEMENT & Gaelle DAIREAUX\n");
			printf ("-h : this help\n-p : fullscreen\n-f : windowed\n");
			exit (0);
		}
		else if (!strcmp (argv[i], "-p"))	// Mode plein cran
			flags = SDL_SWSURFACE | SDL_FULLSCREEN;
		else if (!strcmp (argv[i], "-f"))	// Mode fentre
			flags = SDL_SWSURFACE;
		else
		{	fprintf (stderr, "L'argument %s n'est pas valide.\n", argv[i]);
			exit (1);
		}
	}	
	
	// Initialise SDL
	if (SDL_Init (SDL_INIT_VIDEO) < 0)
	{	fprintf (stderr, "Impossible d'initialiser SDL (%s)\n", SDL_GetError());
		exit (1);
	}
	// Librera les ressources utilises par SDL en quittant
	atexit (SDL_Quit);
	
	// Initialise l'affichage en plein cran ou en fentre en 1024x768x32
	jeu.ecran = SDL_SetVideoMode (1024, 768, 32, flags);
	if (!jeu.ecran)
	{	fprintf (stderr, "Impossible de definir le mode 1024*768*32 (%s)\n", SDL_GetError());
		exit (1);
	}
	// Dfinit le titre de la fentre
	SDL_WM_SetCaption ("Do'SSi Zo'la", "Do'SSi Zo'la");
	
	// Cre un back buffer (une copie de sauvegarde de l'cran en mmoire)
	if (!(jeu.back = SDL_DisplayFormat (jeu.ecran)))
	{ 	fprintf (stderr, "Impossible de crer un back buffer (%s)\n", SDL_GetError ());
		SDL_FreeSurface (jeu.back);
		exit (1);
	}
	
	// Prpare la police de caractre
	if (!(jeu.police.img = SDL_LoadBMP (IMG_POLICE))) ImageErreur (IMG_POLICE);
	jeu.police.rect.x = 0;
	jeu.police.rect.y = 0;
	jeu.police.rect.w = jeu.police.img->w / 37;		// Il y a 37 caracteres dans la police
	jeu.police.rect.h = jeu.police.img->h;
	SDL_SetColorKey (jeu.police.img, SDL_SRCCOLORKEY, SDL_MapRGB (jeu.police.img->format, 255, 0, 255));
	
	// Lance le menu de choix des options du jeu
	Menu (&jeu);
	
	// Lance une partie sur une grille de la taille dfinie
	Jeu (&jeu);
	
	return 0;
}
