void Afficher_Joueur (JOUEUR *j, SDL_Surface *ecran)
{	
	SDL_Surface *img;		// Surface contenant l'animation en cours
	int n;					// Nombre d'images de l'animation en cours
	
	// Si l'animation est l'un des 4 dplacements
	if (j->anim >= COURSB && j->anim <= COURSD)
	{	
		// Place le rectangle source  la bonne position en fonction de l'image  afficher dans l'animation
		j->src.w = j->imgCours->w / j->nCours;
		j->src.h = j->imgCours->h / 4;	
		j->src.x = j->src.w * j->frame;
		j->src.y = j->src.h * j->anim;
		SDL_BlitSurface (j->imgCours, &j->src, ecran, &j->rect);
		return;
	}
	// Autre type d'animation
	else
	{
		if (j->anim == LANCE)	{ img = j->imgLance;	n = j->nLance; }
		if (j->anim == GAGNE)	{ img = j->imgGagne;	n = j->nGagne; }
		if (j->anim == PERDU)	{ img = j->imgPerdu;	n = j->nPerdu; }
		if (j->anim == ATTEND)	{ img = j->imgAttend;	n = j->nAttend; }
		if (j->anim == ATTENDFEU) { img = j->imgAttendFeu; n = j->nAttendFeu; }
		
		// Place le rectangle source  la bonne position en fonction de l'image  afficher dans l'animation
		j->src.w = img->w / n;
		j->src.h = img->h;	
		j->src.x = j->src.w * j->frame;
		j->src.y = 0;
		SDL_BlitSurface (img, &j->src, ecran, &j->rect);
	}
}


// ****************************************************************************
// ******************************** Placer_Trou ********************************
// ****************************************************************************
BOOL Placer_Trou (int x, int y, JOUEUR j[], int n, GRILLE *grille, SDL_Surface *ecran, SDL_Surface *back)
{
	SDL_Rect src, dest, prec;	// Rectangles pour la copie d'image
	int xDep, yDep;				// Coordonnes o la boule dcolle
	int xDest, yDest;			// Coordonnes o la boule s'crase
	int frame;					// Image courante dans l'animation de la boule
	BOOL monte = OUI;			// La boule de feu monte ou descend ?
	SDL_Event evt;				// Pour dtecter un appui sur [ESC]
	
	// Vrifie s'il l'on peut placer le trou
	if (grille->c[x][y] != VIDE) return NON;
	grille->c[x][y] = TROU;

	// Animation du joueur qui lance la boule
	j[n].anim = LANCE;
	j[n].frame = 0;
	frame = -30;
	
	// La boule de feu est centre sur le point du joueur au dpart
	xDest = Case2CoordX (x, *grille) - (j[n].imgFeu->w / j[n].nFeu - grille->imgCase->w) / 2;
	yDest = Case2CoordY (y, *grille) - (j[n].imgFeu->h - grille->imgCase->h) / 2;
	// La boule de feu est centre au centre de la case  dtruire  l'arrive
	xDep = Case2CoordX (j[n].x, *grille) - (j[n].imgFeu->w / j[n].nFeu - grille->imgCase->w) / 2 - 24;
	yDep = Case2CoordY (j[n].y, *grille) - (j[n].imgFeu->h - grille->imgCase->h) / 2 - 16;
	// Rectangle sur l'image dans la squence d'animation de la boule de feu
	src.w = dest.w = j[n].imgFeu->w / j[n].nFeu;
	src.h = dest.h = j[n].imgFeu->h;
	src.x = src.y = 0;
	dest.x = dest.y = 0;
	
	// Boucle de l'animation tant que toutes l'animation de la boule de feu n'a pas t faite
	do
	{
		// Quitte sur un appui de la touche [ESC]
		if (SDL_PollEvent (&evt) && evt.type == SDL_KEYDOWN && evt.key.keysym.sym == SDLK_ESCAPE)
		{	Destructeur (j, grille);
			exit (0);
		}
		
		// Raffiche le dcor sur les anciennes positions
		SDL_BlitSurface (back, &dest, ecran, &dest);
		SDL_BlitSurface (back, &j[n].rect, ecran, &j[n].rect);
		
		// Passe  l'image suivante dans l'animation du joueur (avec une pause entre les frames 0 et 20)
		if ((frame < 0 || frame > 20) && j[n].frame < j[n].nLance - 1) j[n].frame ++;
		
		// Affiche les joueurs avec le plus en avant par dessus l'autre
		Afficher_Joueur (j[n].y <= j[!n].y ? &j[n]  : &j[!n], ecran);
		Afficher_Joueur (j[n].y <= j[!n].y ? &j[!n] : &j[n],  ecran);
		
		// Anime la boule  partir de la frame 0 (on a commenc  -30)
		if (frame >= 0)
		{	
			// Copie l'image de la boule de feu sur l'cran
			prec = dest;
			dest.x = xDep + frame * 2 * (xDest - xDep) / (j[n].nFeu * 2);
			dest.y = yDep + frame * 2 * (yDest - yDep) / (j[n].nFeu * 2);
			SDL_BlitSurface (j[n].imgFeu, &src, ecran, &dest);
			
			// La boule monte ou descend ? affiche l'image suivante ou prcedente dans l'animation
			if (monte) src.x += src.w * BOULE_VITESSE;
			else       src.x -= src.w * BOULE_VITESSE;
			if (src.x >= j[n].nFeu * src.w - src.w * BOULE_VITESSE) monte = NON;
		}

		// Attend d'tre synchronis sur la vitesse d'affichage et rafrachit l'cran
		Attendre_FPS ();
		SDL_UpdateRects (ecran, 1, &j[n].rect);
		if (frame >= 0)
		{	SDL_UpdateRects (ecran, 1, &dest);
			SDL_UpdateRects (ecran, 1, &prec);
		}
	}
	while (++ frame <= j[n].nFeu);
	
	// Rectangle du trou cre
	dest.w = dest.h = grille->imgCase->w;
	dest.x = Case2CoordX (x, *grille);
	dest.y = Case2CoordY (y, *grille);
	
	// Copie l'image du trou sur l'cran et dans le back buffer
	SDL_BlitSurface (grille->imgFond, &dest, ecran, &dest);
	SDL_BlitSurface (grille->imgFond, &dest, back,  &dest);
	SDL_UpdateRects (ecran, 1, &dest);
	
	// Raffiche les joueurs
	Afficher_Joueur (j[n].y < j[!n].y ? &j[n]  : &j[!n], ecran);
	Afficher_Joueur (j[n].y < j[!n].y ? &j[!n] : &j[n],  ecran);
	SDL_UpdateRects (ecran, 1, &dest);
	
	// Fait trembler l'cran
	Secouer_Ecran (j, *grille, ecran);
	
	return OUI;
}


// ****************************************************************************
// ****************************** Deplacer_Joueur *****************************
// ****************************************************************************
BOOL Deplacer_Joueur (int x, int y, JOUEUR j[], int n, GRILLE *grille, SDL_Surface *ecran, SDL_Surface *back)
{
	SDL_Rect prec;				// Rectangle de l'ancienne position du joueur ( effacer)
	SDL_Event evt;				// Pour dtecter un appui sur [ESC]
	int frame;
	
	// Vrifie si l'on peut dplacer le joueur, le dplacement autoris est d'une case autour de sa position
	if (grille->c[x][y] != VIDE ||  j[n].x < x-1 || j[n].x > x+1 || j[n].y < y-1 || j[n].y > y+1) return NON;

	// Choisit la bonne animation pour le joueur
	if (j[n].x > x) j[n].anim = COURSG;
	if (j[n].y > y) j[n].anim = COURSH;
	if (j[n].x < x) j[n].anim = COURSD;
	if (j[n].y < y) j[n].anim = COURSB;
	j[n].frame = 0;
	
	// Animation du joueur qui court d'une case  l'autre
	for (frame = 0; frame < grille->imgCase->w; frame += 2)
	{
		// Quitte sur un appui sur la touche [ESC]
		if (SDL_PollEvent (&evt) && evt.type == SDL_KEYDOWN && evt.key.keysym.sym == SDLK_ESCAPE)
		{	Destructeur (j, grille);
			exit (0);
		}
		
		// Efface l'ancienne position du joueur
		SDL_BlitSurface (back, &j[n].rect, ecran, &j[n].rect);
		
		// Place le rectangle source sur l'image en cours dans l'animation
		j[n].src.x = j[n].frame * j[n].imgCours->w / j[n].nCours;
		
		// Dplace le rectangle destination sur l'cran selon la direction o se dplace le joueur
		prec = j[n].rect;
		if (j[n].x > x) j[n].rect.x -= 2;
		if (j[n].y > y) j[n].rect.y -= 2;
		if (j[n].x < x) j[n].rect.x += 2;
		if (j[n].y < y) j[n].rect.y += 2;
		
		// Affiche les joueurs avec le plus en avant par dessus l'autre
		Afficher_Joueur (j[n].y <= j[!n].y ? &j[n]  : &j[!n], ecran);
		Afficher_Joueur (j[n].y <= j[!n].y ? &j[!n] : &j[n],  ecran);
		
		// Attend d'tre synchronis sur la vitesse d'affichage et rafrachit l'cran
		Attendre_FPS ();
		SDL_UpdateRects (ecran, 1, &j[n].rect);
		SDL_UpdateRects (ecran, 1, &prec);
		
		// Passe  l'image suivante dans l'animation (reboucle si termin)
		if (++ j[n].frame >= j[n].nCours) j[n].frame = 0;
	}
	
	// Libre la place anciennement occupe par le joueur
	grille->c[j[n].x][j[n].y] = VIDE;
	// Modifie ses coordonnes
	j[n].x = x;
	j[n].y = y;
	// Et entre sa nouvelle position dans la grille
	grille->c[j[n].x][j[n].y] = j[n].n;
	
	return OUI;
}


// ****************************************************************************
// ******************************* Placer_Joueur ******************************
// ****************************************************************************
BOOL Placer_Joueur (int x, int y, JOUEUR j[], int n, GRILLE *grille, SDL_Surface *ecran, SDL_Surface *back)
{
	int alpha;					// Valeur de la transparence de l'image du joueur
	
	// Vrifie qu'il y a de la place pour placer le joueur et le place
	if (grille->c[x][y] != VIDE) return NON;
	
	grille->c[x][y] = j[n].n;
	j[n].x = x;
	j[n].y = y;
	
	// Coordonnes du joueur sur l'cran
	j[n].rect.x = Case2CoordX (j[n].x, *grille) - (j[n].imgCours->w / j[n].nCours - grille->imgCase->w) / 2;
	j[n].rect.y = Case2CoordY (j[n].y, *grille) - (j[n].imgCours->h / 4           - grille->imgCase->h) / 2;
	
	// Le joueur apparat progressivement (en diminuant sa transparence)
	for (alpha = 255; alpha >= 0; alpha -= 5)
	{
		// Rgle la transparence du joueur et l'affiche
		SDL_SetAlpha (j[n].imgCours, SDL_SRCALPHA, alpha);
		SDL_BlitSurface (back, &j[n].rect, ecran, &j[n].rect);
		
		// Si c'est au tour du 2e joueur, on raffiche le premier
		if (n == 1) Afficher_Joueur (&j[0], ecran);
		Afficher_Joueur (&j[n], ecran);
		
		// Attend d'tre synchronis sur la vitesse d'affichage et rafrachit l'cran
		Attendre_FPS ();
		SDL_UpdateRects (ecran, 1, &j[n].rect);
	}
	
	return OUI;
}


// ****************************************************************************
// ******************************** Placer_Joueurs ****************************
// ****************************************************************************
void Placer_Joueurs (JOUEUR j[], GRILLE *grille, SDL_Surface *ecran, SDL_Surface *back)
{
	SDL_Event evt;				// Rcupre les infos d'un vnement
	int tour = 0;				// Tour du joueur pour se placer
	int x, y;					// Coordonnes du clic de souris
	
	while (1)
	{
		// Placement d'un joueur dirig par l'ordinateur
		if (j[tour].type == ORDI)
		{	
			Ordi_Place (&x, &y, j[tour], *grille);
			Placer_Joueur (x, y, j, tour, grille, ecran, back);
			
			// Incrmente le tour et quitte si tous les joueurs ont t placs
			if (++ tour == 2) return;
			continue;
		}
		
		if (SDL_PollEvent (&evt))
		switch (evt.type)
		{
			case SDL_KEYDOWN:
				if (evt.key.keysym.sym == SDLK_ESCAPE)	// [Esc] pour quitter
				{	Destructeur (j, grille);
					exit (0);
				}
				break;
			
			case SDL_MOUSEBUTTONDOWN:
				if (evt.button.button == 1)				// Bouton gauche enfonc
				{	
					x = Clic2CaseX (evt.button.x, *grille);	// Rcupre les coords
					y = Clic2CaseY (evt.button.y, *grille);	// de la case clique
					
					// S'il s'agit bien d'une case de la grille
					if (x != -1 && y != -1)
						// Si le joueur a put tre plac
						if (Placer_Joueur (x, y, j, tour, grille, ecran, back))
							// Incrmente le tour et quitte si tous les joueurs ont t placs
							if (++ tour == 2) return;
				}
		}
	}
}


// ****************************************************************************
// ****************************** Constructeur ********************************
// ****************************************************************************
void Constructeur (JOUEUR j[], GRILLE *grille, BOOL c, SDL_Surface *ecran, SDL_Surface *back)
{
	SDL_Surface *img;			// Surface contenant temporairement une image
	SDL_Rect src, dest;			// Rectangles de copie d'image
	int x, y;
	
	// Crer une surface qui contiendra l'arrire plan
	if (!(grille->imgFond = SDL_DisplayFormat (ecran)))
	{	fprintf (stderr, "Impossible de crer le fond d'cran (%s)\n", SDL_GetError ());
		SDL_FreeSurface (grille->imgFond);
		exit (1);
	}
	
	// Charge l'image de fond ...
	switch (Alea (1, 6))
	{
		case 1:	if (!(img = SDL_LoadBMP (IMG_FOND1))) ImageErreur (IMG_FOND1);	break;
		case 2:	if (!(img = SDL_LoadBMP (IMG_FOND2))) ImageErreur (IMG_FOND2);	break;
		case 3:	if (!(img = SDL_LoadBMP (IMG_FOND3))) ImageErreur (IMG_FOND3);	break;
		case 4:	if (!(img = SDL_LoadBMP (IMG_FOND4))) ImageErreur (IMG_FOND4);	break;
		case 5:	if (!(img = SDL_LoadBMP (IMG_FOND5))) ImageErreur (IMG_FOND5);	break;
		default: if (!(img = SDL_LoadBMP (IMG_FOND1))) ImageErreur (IMG_FOND1);
	}
	
	// ... et l'affiche en mosaique dans l'arrire plan
	src.x = src.y = 0;
	
	for (dest.y = 0; dest.y < grille->imgFond->h; dest.y += dest.h)
		for (dest.x = 0; dest.x < grille->imgFond->w; dest.x += dest.w)
		{	
			src.w = dest.w = img->w;
			src.h = dest.h = img->h;
			SDL_BlitSurface (img, &src, grille->imgFond, &dest);
		}
	
	// Libre l'image de fond et copie l'arrire plan dans le back buffer
	SDL_FreeSurface (img);
	SDL_BlitSurface (grille->imgFond, NULL, back, NULL);
	
	// Charge les images dans des surfaces et teste si elles sont prsentes
	if (!(grille->imgCase	= SDL_LoadBMP (IMG_CASE)))			ImageErreur (IMG_CASE);
	if (!(j[c].imgFeu		= SDL_LoadBMP (IMG_FEU)))			ImageErreur (IMG_FEU);
	if (!(j[!c].imgFeu		= SDL_LoadBMP (IMG_FEU)))			ImageErreur (IMG_FEU);
	if (!(j[c].imgCours		= SDL_LoadBMP (IMG_J1COURS)))		ImageErreur (IMG_J1COURS);
	if (!(j[!c].imgCours	= SDL_LoadBMP (IMG_J2COURS)))		ImageErreur (IMG_J2COURS);
	if (!(j[c].imgLance		= SDL_LoadBMP (IMG_J1LANCE)))		ImageErreur (IMG_J1LANCE);
	if (!(j[!c].imgLance	= SDL_LoadBMP (IMG_J2LANCE)))		ImageErreur (IMG_J2LANCE);
	if (!(j[c].imgGagne		= SDL_LoadBMP (IMG_J1GAGNE)))		ImageErreur (IMG_J1GAGNE);
	if (!(j[!c].imgGagne	= SDL_LoadBMP (IMG_J2GAGNE)))		ImageErreur (IMG_J2GAGNE);
	if (!(j[c].imgPerdu		= SDL_LoadBMP (IMG_J1PERDU)))		ImageErreur (IMG_J1PERDU);
	if (!(j[!c].imgPerdu	= SDL_LoadBMP (IMG_J2PERDU)))		ImageErreur (IMG_J2PERDU);
	if (!(j[c].imgAttend	= SDL_LoadBMP (IMG_J1ATTEND)))		ImageErreur (IMG_J1ATTEND);
	if (!(j[!c].imgAttend	= SDL_LoadBMP (IMG_J2ATTEND)))		ImageErreur (IMG_J2ATTEND);
	if (!(j[c].imgAttendFeu	= SDL_LoadBMP (IMG_J1ATTENDFEU)))	ImageErreur (IMG_J1ATTENDFEU);
	if (!(j[!c].imgAttendFeu = SDL_LoadBMP (IMG_J2ATTENDFEU)))	ImageErreur (IMG_J2ATTENDFEU);
	
	// Dfinit le nombres d'images pour les animations
	j[c].nFeu       = j[!c].nFeu       = 60;
	j[c].nCours     = j[!c].nCours     = 32;
	j[c].nLance     = j[!c].nLance     = 36;
	j[c].nGagne     = j[!c].nGagne     = 70;
	j[c].nPerdu     = j[!c].nPerdu     = 40;
	j[c].nAttend    = j[!c].nAttend    = 60;
	j[c].nAttendFeu = j[!c].nAttendFeu = 40;
	
	// Animation courante et image  afficher
	j[c].anim = j[!c].anim = COURSB;
	j[c].frame = j[!c].frame = 0;
	
	// Dfinit la ceur qui dtermine la transparence (le violet)
	SDL_SetColorKey (j[c].imgFeu,	 	SDL_SRCCOLORKEY, SDL_MapRGB (j[c].imgFeu->format,		255, 0, 255));
	SDL_SetColorKey (j[!c].imgFeu,	 	SDL_SRCCOLORKEY, SDL_MapRGB (j[!c].imgFeu->format,		255, 0, 255));
	SDL_SetColorKey (j[c].imgCours,	 	SDL_SRCCOLORKEY, SDL_MapRGB (j[c].imgCours->format,		255, 0, 255));
	SDL_SetColorKey (j[!c].imgCours, 	SDL_SRCCOLORKEY, SDL_MapRGB (j[!c].imgCours->format,	255, 0, 255));
	SDL_SetColorKey (j[c].imgLance,	 	SDL_SRCCOLORKEY, SDL_MapRGB (j[c].imgLance->format,		255, 0, 255));
	SDL_SetColorKey (j[!c].imgLance, 	SDL_SRCCOLORKEY, SDL_MapRGB (j[!c].imgLance->format,	255, 0, 255));
	SDL_SetColorKey (j[c].imgGagne,	 	SDL_SRCCOLORKEY, SDL_MapRGB (j[c].imgGagne->format,		255, 0, 255));
	SDL_SetColorKey (j[!c].imgGagne, 	SDL_SRCCOLORKEY, SDL_MapRGB (j[!c].imgGagne->format,	255, 0, 255));
	SDL_SetColorKey (j[c].imgPerdu,	 	SDL_SRCCOLORKEY, SDL_MapRGB (j[c].imgPerdu->format,		255, 0, 255));
	SDL_SetColorKey (j[!c].imgPerdu, 	SDL_SRCCOLORKEY, SDL_MapRGB (j[!c].imgPerdu->format,	255, 0, 255));
	SDL_SetColorKey (j[c].imgAttend, 	SDL_SRCCOLORKEY, SDL_MapRGB (j[c].imgAttend->format,	255, 0, 255));
	SDL_SetColorKey (j[!c].imgAttend, 	SDL_SRCCOLORKEY, SDL_MapRGB (j[!c].imgAttend->format,	255, 0, 255));
	SDL_SetColorKey (j[c].imgAttendFeu,	SDL_SRCCOLORKEY, SDL_MapRGB (j[c].imgAttendFeu->format,	255, 0, 255));
	SDL_SetColorKey (j[!c].imgAttendFeu, SDL_SRCCOLORKEY, SDL_MapRGB (j[!c].imgAttendFeu->format, 255, 0, 255));
	
	// Dfinit la transparence des cases et de la boule de feu
	SDL_SetAlpha (grille->imgCase,	SDL_SRCALPHA, 100);
	SDL_SetAlpha (j[c].imgFeu,		SDL_SRCALPHA, 64);
	SDL_SetAlpha (j[!c].imgFeu,		SDL_SRCALPHA, 64);
	SDL_SetAlpha (j[c].imgPerdu,	SDL_SRCALPHA, 100);
	SDL_SetAlpha (j[!c].imgPerdu,	SDL_SRCALPHA, 100);
		
	// Dfinit la taille des rectangles des joueurs
	j[c].rect.w = j[c].imgCours->w / j[c].nCours;
	j[c].rect.h = j[c].imgCours->h / 4;
	j[!c].rect.w = j[!c].imgCours->w / j[c].nCours;
	j[!c].rect.h = j[!c].imgCours->h / 4;
	
	// Remplit la grille de cases vides
	for (x = 0; x < grille->xc; x ++)
		for (y = 0; y < grille->yc; y ++)
			grille->c [x][y] = VIDE;
	
	// Grille positionne au milieu de l'cran
	grille->tc = grille->imgCase->w;
	grille->x1 = (ecran->w - grille->tc * grille->xc) / 2;
	grille->y1 = (ecran->h - grille->tc * grille->yc) / 2;
	grille->x2 = ecran->w - grille->x1;
	grille->y2 = ecran->h - grille->y1;
	
	// Copie les cases de la grille dans le back buffer
	dest.w = dest.h = grille->tc;
	
	for (dest.x = grille->x1; dest.x < grille->x2; dest.x += dest.w)
		for (dest.y = grille->y1; dest.y < grille->y2; dest.y += dest.h)
			SDL_BlitSurface (grille->imgCase, NULL, back, &dest);

	// Met  jour l'cran entier
	SDL_BlitSurface (back, NULL, ecran, NULL);
	SDL_UpdateRect (ecran, 0, 0, ecran->w, ecran->h);
}


// ****************************************************************************
// ****************************** Destructeur *********************************
// ****************************************************************************
void Destructeur (JOUEUR j[], GRILLE *grille)
{
	// Enlve les images de la mmoire
	SDL_FreeSurface (grille->imgCase);
	SDL_FreeSurface (grille->imgFond);  
	SDL_FreeSurface (j[0].imgFeu);
	SDL_FreeSurface (j[1].imgFeu);
	SDL_FreeSurface (j[0].imgCours);
	SDL_FreeSurface (j[1].imgCours);
	SDL_FreeSurface (j[0].imgLance);
	SDL_FreeSurface (j[1].imgLance);
	SDL_FreeSurface (j[0].imgGagne);
	SDL_FreeSurface (j[1].imgGagne);
	SDL_FreeSurface (j[0].imgPerdu);
	SDL_FreeSurface (j[1].imgPerdu);
}


// ****************************************************************************
// ****************************** Tester_Perdu ********************************
// ****************************************************************************
BOOL Tester_Perdu (JOUEUR j, GRILLE grille)
{
	// Teste si les 8 cases entourants le joueur existent et si elles sont vides
	if (j.x-1 >= 0 && j.y-1 >= 0        && grille.c[j.x-1][j.y-1] == VIDE) return NON;
	if (j.x-1 >= 0                      && grille.c[j.x-1][j.y]   == VIDE) return NON;
	if (j.x-1 >= 0 && j.y+1 < grille.yc && grille.c[j.x-1][j.y+1] == VIDE) return NON;
	
	if (j.y-1 >= 0        && grille.c[j.x][j.y-1] == VIDE) return NON;
	if (j.y+1 < grille.yc && grille.c[j.x][j.y+1] == VIDE) return NON;
	
	if (j.x+1 < grille.xc && j.y-1 >= 0        && grille.c[j.x+1][j.y-1] == VIDE) return NON;
	if (j.x+1 < grille.xc                      && grille.c[j.x+1][j.y]   == VIDE) return NON;
	if (j.x+1 < grille.xc && j.y+1 < grille.yc && grille.c[j.x+1][j.y+1] == VIDE) return NON;
	
	return OUI;
}


// ****************************************************************************
// ************************** Animer_Perdant_Gagnant **************************
// ****************************************************************************
void Animer_Perdant_Gagnant (JOUEUR *j1, JOUEUR *j2, SDL_Surface *ecran, SDL_Surface *back)
{
	SDL_Rect prec;				// Prcdente position du joueur qui saute
	SDL_Event evt;				// Pour dtecter un appui sur [ESC]
	
	// Rajuste le rectangles du joueur qui se transforme en ange
	j2->rect.w = j2->imgPerdu->w / j2->nPerdu;
	j2->rect.h = j2->imgPerdu->h;
	j2->rect.x -= 32;
	j2->rect.y -= 16;
	// Rgle les animations
	j1->anim = GAGNE;
	j2->anim = PERDU;
	j1->frame = 0;
	j2->frame = 20;
	
	// Animation des joueurs
	while (1)
	{
		// Quitte sur un appui de la touche [ESC]
		if (SDL_PollEvent (&evt) && evt.type == SDL_KEYDOWN && evt.key.keysym.sym == SDLK_ESCAPE)
			exit (0);
		
		// Efface les anciennes positions des joueurs
		SDL_BlitSurface (back, &j1->rect, ecran, &j1->rect);
		SDL_BlitSurface (back, &j2->rect, ecran, &j2->rect);
		
		// Impulsion du saut prilleux
		prec = j1->rect;
		if (j1->frame > 5  && j1->frame <= 17) j1->rect.y -= 4;
		if (j1->frame > 17 && j1->frame < 30)  j1->rect.y += 4;
		
		// Impulsion des ailes
		if (j2->frame > 20 && j2->frame < 32)  j2->rect.y -= 4;
		else j2->rect.y --;
		
		// Affiche les joueurs anims
		Afficher_Joueur (j1, ecran);
		Afficher_Joueur (j2, ecran);
		
		// Attend d'tre synchronis sur la vitesse d'affichage et rafrachit l'cran
		Attendre_FPS ();
		SDL_UpdateRects (ecran, 1, &j1->rect);
		SDL_UpdateRects (ecran, 1, &j2->rect);
		SDL_UpdateRects (ecran, 1, &prec);
		
		// Quitte si l'ange est en haut de l'cran
		if (j2->rect.y <= 0) return;
		
		// Passe  l'image suivante dans l'animation et reboucle si besoin
		if (++j1->frame >= j1->nGagne) j1->frame = 0;
		if (++j2->frame >= j2->nPerdu) j2->frame = 0;
	}
}


// ****************************************************************************
// *********************************** Jeu ************************************
// ****************************************************************************
void Jeu (JEU *jeu)
{
	SDL_Event evt;			        // Sert  recueillir des infos sur les vnements
	int x, y;				        // Coordonnes des clics de souris
	ACTION action = J1_DEPLACE;		// Action du jeu en cours
	GRILLE grille = {jeu->XGRILLE, jeu->YGRILLE};	// Grille du jeu (initialise dimensions)
	JOUEUR j[2] = {{J1, jeu->J1TYPE, jeu->J1NIVO}, {J2, jeu->J2TYPE, jeu->J2NIVO}};	// Joueurs
	BOOL bis = OUI;					// Si l'option double est active: compteur d'action double
	int n;
	
	Constructeur (j, &grille, jeu->J1ROUGE, jeu->ecran, jeu->back);	// Charge les images, la grille, ...
	Placer_Joueurs (j, &grille, jeu->ecran, jeu->back);	// Pour choisir la position de dpart
	
	// Boucle de jeu principale
	while (1)
	{
		// Test si l'un des joueurs  perdu
		if (((action == J1_DEPLACE || action == J2_TROU) && Tester_Perdu (j[0], grille)) || ((action == J2_DEPLACE || action == J1_TROU) && Tester_Perdu (j[1], grille)))
		{	
			// Quel joueur a perdu ?
			if ((action == J1_DEPLACE || action == J2_TROU) && Tester_Perdu (j[0], grille)) n = 0;
			else n = 1;
			
			// Le perdant se suicide
			grille.c[j[n].x][j[n].y] = VIDE;
			Placer_Trou (j[n].x, j[n].y, j, n, &grille, jeu->ecran, jeu->back);
			
			// Anime les joueurs (le gagnant danse et le perdant s'envole en ange)
			Animer_Perdant_Gagnant (&j[!n], &j[n], jeu->ecran, jeu->back);
			
			// Copie les joueurs dans le back buffer et affiche qui a gagn
			Afficher_Joueur (&j[0], jeu->back);
			Afficher_Joueur (&j[1], jeu->back);
			Afficher_Chaine ((jeu->ecran->w - strlen (TXT_J2GAGNE) * jeu->police.rect.w) / 2, (jeu->ecran->h - jeu->police.rect.h) / 2, CARACT_VITESSE, TXT_J2GAGNE, jeu->police, jeu->ecran, jeu->back);
			
			// Quitte si l'on appui sur [ESC]
			while (1)
			if (SDL_WaitEvent (&evt) && evt.type == SDL_KEYDOWN && evt.key.keysym.sym == SDLK_ESCAPE)
			{	Destructeur (j, &grille);
				exit (0);
			}
		}
		
		// Si c'est  un humain de jouer, anime le joueur en attendant un vnement
		while ((action / 2 == 0 && j[0].type == HUMAIN) || (action / 2 == 1 && j[1].type == HUMAIN))
		{	
			// Affiche les joueurs sur l'cran
			SDL_BlitSurface (jeu->back, &j[0].rect, jeu->ecran, &j[0].rect);
			SDL_BlitSurface (jeu->back, &j[1].rect, jeu->ecran, &j[1].rect);
			Afficher_Joueur (j[0].y > j[1].y ? &j[1] : &j[0], jeu->ecran);
			Afficher_Joueur (j[0].y > j[1].y ? &j[0] : &j[1], jeu->ecran);
			SDL_UpdateRects (jeu->ecran, 1, &j[0].rect);
			SDL_UpdateRects (jeu->ecran, 1, &j[1].rect);
			Attendre_FPS ();
			
			if (SDL_PollEvent (&evt))		// Attend un vnement
			{
				if (evt.type == SDL_KEYDOWN && evt.key.keysym.sym == SDLK_ESCAPE)	// [Esc] pour quitter
				{	Destructeur (j, &grille);
					exit (0);
				}
				
				if (evt.type == SDL_MOUSEBUTTONDOWN && evt.button.button == 1)		// Clic bouton gauche
				{
					// Rcupre les coordonnes de la case clique, -1 si en dehors de la grille
					x = Clic2CaseX (evt.button.x, grille);
					y = Clic2CaseY (evt.button.y, grille);
					
					// Arrte l'attente d'vnement si le clic est sur une case de la grille
					if (x != -1 && y != -1) break;
				}
			}
			
			// Rgle les animations pour le joueur qui attend de jouer et celui qui ne bouge pas
			switch (action)
			{
				case J1_DEPLACE:	j[0].anim = ATTEND;
									if (++ j[0].frame >= j[0].nAttend) j[0].frame = 0;
									j[1].anim = COURSB;
									j[1].frame = 0;
									break;
				
				case J1_TROU:		j[0].anim = ATTENDFEU;
									if (++ j[0].frame >= j[0].nAttendFeu) j[0].frame = 0;
									j[1].anim = COURSB;
									j[1].frame = 0;
									break;
				
				case J2_DEPLACE:	j[1].anim = ATTEND;
									if (++ j[1].frame >= j[1].nAttend) j[1].frame = 0;
									j[0].anim = COURSB;
									j[0].frame = 0;
									break;
				
				case J2_TROU:		j[1].anim = ATTENDFEU;
									if (++ j[1].frame >= j[1].nAttendFeu) j[1].frame = 0;
									j[0].anim = COURSB;
									j[0].frame = 0;
									break;
			}
		}
		
		// Fait jouer les joueurs dirigs par l'ordinateur sinon anime le joueur qui attend
		switch (action)
		{
			case J1_DEPLACE:
				if (j[0].type == ORDI) Ordi_Deplace (&x, &y, j, 0, grille);
				
				if (Deplacer_Joueur (x, y, j, 0, &grille, jeu->ecran, jeu->back))
				{	
					if (jeu->DOUBLE_DEPLACE && !(bis = !bis)) break;
					action = J1_TROU;
				}
				break;
			
			case J1_TROU:
				if (j[0].type == ORDI) Ordi_Trou (&x, &y, j, 1, grille);
				
				if (Placer_Trou (x, y, j, 0, &grille, jeu->ecran, jeu->back))
				{	
					if (jeu->DOUBLE_BOULEFEU && !(bis = !bis)) break;
					action = J2_DEPLACE;
					// Place alatoirement une boule si l'option est active
					if (jeu->INTEMP) Placer_Trou (Alea (0, grille.xc), Alea (0, grille.yc), j, 1, &grille, jeu->ecran, jeu->back);
				}
				break;
			
			case J2_DEPLACE:
				if (j[1].type == ORDI) Ordi_Deplace (&x, &y, j, 1, grille);
				
				if (Deplacer_Joueur (x, y, j, 1, &grille, jeu->ecran, jeu->back))
				{	
					if (jeu->DOUBLE_DEPLACE && !(bis = !bis)) break;
					action = J2_TROU;
					break;
				}
			
			case J2_TROU:
				if (j[1].type == ORDI) Ordi_Trou (&x, &y, j, 0, grille);
				
				if (Placer_Trou (x, y, j, 1, &grille, jeu->ecran, jeu->back))
				{	
					if (jeu->DOUBLE_BOULEFEU && !(bis = !bis)) break;
					action = J1_DEPLACE;
					// Place alatoirement une boule si l'option est active
					if (jeu->INTEMP) Placer_Trou (Alea (0, grille.xc), Alea (0, grille.yc), j, 0, &grille, jeu->ecran, jeu->back);
					break;
				}
		}
	}
}
